<?php
/**
 * @copyright   Copyright (C) 2005 - 2013 Michael Richey. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;
class TagParser {    
    private $identifier;
    private $options = array(
        'bracket'=>'{}',
        'selfclosing'=>false
    );
    public $tags = array();

    public function __construct($identifier,$options=false) {
        $this->identifier = $identifier;
        if(is_array($options)) {
            foreach($options as $key=>$option) $this->options[$key]=$option;
        }
        if(strlen($this->options['bracket']) == 1) $this->options['bracket'] = $this->options['bracket'].$this->options['bracket'];
    }
    
    public function parse($content) {
        $regex ='/'.$this->bracket('open').$this->identifier.'(?<params>.*?)'.$this->bracket('close');
        if(!$this->options['selfclosing']) $regex.='(?<content>.*?)(?<closetag>'.$this->bracket('open').'\/'.$this->identifier.$this->bracket('close').')';
        $regex.='/';
        $matches = array();
        preg_match_all($regex,$content,$matches);
        foreach($matches[0] as $key=>$match) {
            $this->tags[$key]=new TagParserItem($match);
            if(strlen($matches['params'][$key])) {
                $params = new DOMDocument;
                $params->loadHTML('<br '.trim($matches['params'][$key]).' />');
                $paramsxml = simplexml_import_dom($params->documentElement);
                if(count($paramsxml->body->br->attributes())) {
                    $this->tags[$key]->params = array();
                    foreach($paramsxml->body->br->attributes() as $attribute=>$value) {
                        $this->tags[$key]->params[$attribute]=(strlen($value->__toString())?$value->__toString():true);
                    }
                }
            }
            if(!$this->options['selfclosing']) $this->tags[$key]->content = $matches['content'][$key];
        }
    }
    
    private function bracket($type='open') {
        switch($type) {
            case 'open':
                $return = preg_quote($this->options['bracket'][0]);
                break;
            case 'close':
                $return = preg_quote($this->options['bracket'][1]);
                break;
        }
        return $return;
    }
}
class TagParserItem {
    public $tag;
    public $params = false;
    public $content = false;
    public function __construct($tag) {
        $this->tag = $tag;
    }
    public function getParam($name,$default) {
        return isset($this->params[$name])?$this->params[$name]:$default;
    }
    public function getContent($default='') {
        return $this->content?$this->content:$default;
    }
    public function replace($content,$replacement='') {
        $regex='/'.preg_quote($this->tag,'/').'/';
        return preg_replace($regex,$replacement,$content,1);
    }
}
